@@ -38,6 +38,7 @@
 #include <fstream>
 #include <sstream>
 #include <boost/algorithm/string.hpp>
+#include <boost/lexical_cast.hpp>
 #ifndef SHIBSP_LITE
 # include <saml/exceptions.h>
 # include <saml/saml2/metadata/MetadataProvider.h>
@@ -85,7 +86,7 @@ namespace shibsp {
             if (mayRedirect)
                 redirectErrors = settings.first->getString("redirectErrors");
         }
-        catch (exception& ex) {
+        catch (const exception& ex) {
             log.error(ex.what());
         }
 
@@ -152,6 +153,8 @@ namespace shibsp {
         app.clearHeader(request, "Shib-Cookie-Name", "HTTP_SHIB_COOKIE_NAME");
         app.clearHeader(request, "Shib-Session-ID", "HTTP_SHIB_SESSION_ID");
         app.clearHeader(request, "Shib-Session-Index", "HTTP_SHIB_SESSION_INDEX");
+        app.clearHeader(request, "Shib-Session-Expires", "HTTP_SHIB_SESSION_EXPIRES");
+        app.clearHeader(request, "Shib-Session-Inactivity", "HTTP_SHIB_SESSION_INACTIVITY");
         app.clearHeader(request, "Shib-Identity-Provider", "HTTP_SHIB_IDENTITY_PROVIDER");
         app.clearHeader(request, "Shib-Authentication-Method", "HTTP_SHIB_AUTHENTICATION_METHOD");
         app.clearHeader(request, "Shib-Authentication-Instant", "HTTP_SHIB_AUTHENTICATION_INSTANT");
@@ -171,6 +174,13 @@ namespace shibsp {
 
         const URLEncoder* encoder = XMLToolingConfig::getConfig().getURLEncoder();
 
+        // Default delimiter is semicolon but is now configurable.
+        pair<bool,const char*> delim = settings.first->getString("attributeValueDelimiter");
+        if (enc.first || !delim.first) {
+            delim.second = ";";
+        }
+        size_t delim_len = strlen(delim.second);
+
         pair<bool,bool> exportDups = settings.first->getBool("exportDuplicateValues");
         const multimap<string,const Attribute*>& attributes = session->getIndexedAttributes();
 
@@ -183,18 +193,18 @@ namespace shibsp {
                 const vector<string>& vals = a->second->getSerializedValues();
                 for (vector<string>::const_iterator v = vals.begin(); v != vals.end(); ++v) {
                     if (!header.empty())
-                        header += ';';
+                        header += delim.second;
                     if (enc.first) {
                         // If URL-encoding, any semicolons will get escaped anyway.
                         header += encoder->encode(v->c_str());
                     }
                     else {
-                        string::size_type pos = v->find_first_of(';', string::size_type(0));
+                        string::size_type pos = v->find(delim.second, string::size_type(0));
                         if (pos != string::npos) {
                             string value(*v);
-                            for (; pos != string::npos; pos = value.find_first_of(';', pos)) {
+                            for (; pos != string::npos; pos = value.find(delim.second, pos)) {
                                 value.insert(pos, "\\");
-                                pos += 2;
+                                pos += delim_len + 1;
                             }
                             header += value;
                         }
@@ -221,18 +231,18 @@ namespace shibsp {
                 string header;
                 for (set<string>::const_iterator v = deduped->second.begin(); v != deduped->second.end(); ++v) {
                     if (!header.empty())
-                        header += ';';
+                        header += delim.second;
                     if (enc.first) {
                         // If URL-encoding, any semicolons will get escaped anyway.
                         header += encoder->encode(v->c_str());
                     }
                     else {
-                        string::size_type pos = v->find_first_of(';', string::size_type(0));
+                        string::size_type pos = v->find(delim.second, string::size_type(0));
                         if (pos != string::npos) {
                             string value(*v);
-                            for (; pos != string::npos; pos = value.find_first_of(';', pos)) {
+                            for (; pos != string::npos; pos = value.find(delim.second, pos)) {
                                 value.insert(pos, "\\");
-                                pos += 2;
+                                pos += delim_len + 1;
                             }
                             header += value;
                         }
@@ -280,46 +290,6 @@ ServiceProvider::~ServiceProvider()
 {
 }
 
-#ifndef SHIBSP_LITE
-SecurityPolicyProvider* ServiceProvider::getSecurityPolicyProvider(bool required) const
-{
-    if (required)
-        throw ConfigurationException("No SecurityPolicyProvider available.");
-    return nullptr;
-}
-#endif
-
-Remoted* ServiceProvider::regListener(const char* address, Remoted* listener)
-{
-    Remoted* ret = nullptr;
-    map<string,Remoted*>::const_iterator i = m_listenerMap.find(address);
-    if (i != m_listenerMap.end())
-        ret = i->second;
-    m_listenerMap[address] = listener;
-    Category::getInstance(SHIBSP_LOGCAT ".ServiceProvider").info("registered remoted message endpoint (%s)",address);
-    return ret;
-}
-
-bool ServiceProvider::unregListener(const char* address, Remoted* current, Remoted* restore)
-{
-    map<string,Remoted*>::const_iterator i = m_listenerMap.find(address);
-    if (i != m_listenerMap.end() && i->second == current) {
-        if (restore)
-            m_listenerMap[address] = restore;
-        else
-            m_listenerMap.erase(address);
-        Category::getInstance(SHIBSP_LOGCAT ".ServiceProvider").info("unregistered remoted message endpoint (%s)",address);
-        return true;
-    }
-    return false;
-}
-
-Remoted* ServiceProvider::lookupListener(const char *address) const
-{
-    map<string,Remoted*>::const_iterator i = m_listenerMap.find(address);
-    return (i == m_listenerMap.end()) ? nullptr : i->second;
-}
-
 pair<bool,long> ServiceProvider::doAuthentication(SPRequest& request, bool handler) const
 {
 #ifdef _DEBUG
@@ -334,6 +304,9 @@ pair<bool,long> ServiceProvider::doAuthentication(SPRequest& request, bool handl
         RequestMapper::Settings settings = request.getRequestSettings();
         app = &(request.getApplication());
 
+        string appid(string("[") + app->getId() + "]");
+        xmltooling::NDC ndc(appid.c_str());
+
         // If not SSL, check to see if we should block or redirect it.
         if (!request.isSecure()) {
             pair<bool,const char*> redirectToSSL = settings.first->getString("redirectToSSL");
@@ -392,10 +365,10 @@ pair<bool,long> ServiceProvider::doAuthentication(SPRequest& request, bool handl
         try {
             session = request.getSession(true, false, false);   // don't cache it
         }
-        catch (exception& e) {
+        catch (const exception& e) {
             log.warn("error during session lookup: %s", e.what());
             // If it's not a retryable session failure, we throw to the outer handler for reporting.
-            if (dynamic_cast<opensaml::RetryableProfileException*>(&e) == nullptr)
+            if (dynamic_cast<const opensaml::RetryableProfileException*>(&e) == nullptr)
                 throw;
         }
 
@@ -438,7 +411,7 @@ pair<bool,long> ServiceProvider::doAuthentication(SPRequest& request, bool handl
                 initiator=app->getSessionInitiatorById(requireSessionWith.second);
                 if (!initiator) {
                     throw ConfigurationException(
-                        "No session initiator found with id ($1), check requireSessionWith command.", params(1, requireSessionWith.second)
+                        "No session initiator found with id ($1), check requireSessionWith setting.", params(1, requireSessionWith.second)
                         );
                 }
             }
@@ -470,7 +443,7 @@ pair<bool,long> ServiceProvider::doAuthentication(SPRequest& request, bool handl
         log.debug("doAuthentication succeeded");
         return make_pair(false,0L);
     }
-    catch (exception& e) {
+    catch (const exception& e) {
         request.log(SPRequest::SPError, e.what());
         TemplateParameters tp(&e);
         tp.m_map["requestURL"] = targetURL.substr(0,targetURL.find('?'));
@@ -494,6 +467,9 @@ pair<bool,long> ServiceProvider::doAuthorization(SPRequest& request) const
         RequestMapper::Settings settings = request.getRequestSettings();
         app = &(request.getApplication());
 
+        string appid(string("[") + app->getId() + "]");
+        xmltooling::NDC ndc(appid.c_str());
+
         // Three settings dictate how to proceed.
         pair<bool,const char*> authType = settings.first->getString("authType");
         pair<bool,bool> requireSession = settings.first->getBool("requireSession");
@@ -513,7 +489,7 @@ pair<bool,long> ServiceProvider::doAuthorization(SPRequest& request) const
                 if (session)
                     slocker.assign(session, false); // assign to lock popper
             }
-            catch (exception& e) {
+            catch (const exception& e) {
                 log.warn("unable to obtain session to pass to access control provider: %s", e.what());
             }
 
@@ -540,7 +516,7 @@ pair<bool,long> ServiceProvider::doAuthorization(SPRequest& request) const
             return make_pair(true, request.returnDecline());
         }
     }
-    catch (exception& e) {
+    catch (const exception& e) {
         request.log(SPRequest::SPError, e.what());
         TemplateParameters tp(&e, nullptr, session);
         tp.m_map["requestURL"] = targetURL.substr(0,targetURL.find('?'));
@@ -564,12 +540,15 @@ pair<bool,long> ServiceProvider::doExport(SPRequest& request, bool requireSessio
         RequestMapper::Settings settings = request.getRequestSettings();
         app = &(request.getApplication());
 
+        string appid(string("[") + app->getId() + "]");
+        xmltooling::NDC ndc(appid.c_str());
+
         try {
             session = request.getSession(false, false, false);  // ignore timeout and do not cache
             if (session)
                 slocker.assign(session, false); // assign to lock popper
         }
-        catch (exception& e) {
+        catch (const exception& e) {
             log.warn("unable to obtain session to export to request: %s", e.what());
         	// If we have to have a session, then this is a fatal error.
         	if (requireSession)
@@ -587,6 +566,8 @@ pair<bool,long> ServiceProvider::doExport(SPRequest& request, bool requireSessio
         app->setHeader(request, "Shib-Application-ID", app->getId());
         app->setHeader(request, "Shib-Session-ID", session->getID());
 
+        const PropertySet* sessionProps = app->getPropertySet("Sessions");
+
         // Check for export of "standard" variables.
         // A 3.0 release would switch this default to false and rely solely on the
         // Assertion extractor plugin and ship out of the box with the same defaults.
@@ -609,6 +590,12 @@ pair<bool,long> ServiceProvider::doExport(SPRequest& request, bool requireSessio
             hval = session->getSessionIndex();
             if (hval)
                 app->setHeader(request, "Shib-Session-Index", hval);
+
+            app->setHeader(request, "Shib-Session-Expires", boost::lexical_cast<string>(session->getExpiration()).c_str());
+            pair<bool,unsigned int> timeout = sessionProps ? sessionProps->getUnsignedInt("timeout") : pair<bool,unsigned int>(false, 0);
+            if (timeout.first && timeout.second > 0) {
+                app->setHeader(request, "Shib-Session-Inactivity", boost::lexical_cast<string>(session->getLastAccess() + timeout.second).c_str());
+            }
         }
 
         // Check for export of algorithmically-derived portion of cookie names.
@@ -621,8 +608,7 @@ pair<bool,long> ServiceProvider::doExport(SPRequest& request, bool requireSessio
         // Maybe export the assertion keys.
         pair<bool,bool> exp = settings.first->getBool("exportAssertion");
         if (exp.first && exp.second) {
-            const PropertySet* sessions = app->getPropertySet("Sessions");
-            pair<bool,const char*> exportLocation = sessions ? sessions->getString("exportLocation") : pair<bool,const char*>(false,nullptr);
+            pair<bool,const char*> exportLocation = sessionProps ? sessionProps->getString("exportLocation") : pair<bool,const char*>(false,nullptr);
             if (!exportLocation.first)
                 log.warn("can't export assertions without an exportLocation Sessions property");
             else {
@@ -651,7 +637,7 @@ pair<bool,long> ServiceProvider::doExport(SPRequest& request, bool requireSessio
 
         return make_pair(false,0L);
     }
-    catch (exception& e) {
+    catch (const exception& e) {
         request.log(SPRequest::SPError, e.what());
         TemplateParameters tp(&e, nullptr, session);
         tp.m_map["requestURL"] = targetURL.substr(0,targetURL.find('?'));
@@ -673,6 +659,9 @@ pair<bool,long> ServiceProvider::doHandler(SPRequest& request) const
         RequestMapper::Settings settings = request.getRequestSettings();
         app = &(request.getApplication());
 
+        string appid(string("[") + app->getId() + "]");
+        xmltooling::NDC ndc(appid.c_str());
+
         // If not SSL, check to see if we should block or redirect it.
         if (!request.isSecure()) {
             pair<bool,const char*> redirectToSSL = settings.first->getString("redirectToSSL");
@@ -736,13 +725,13 @@ pair<bool,long> ServiceProvider::doHandler(SPRequest& request) const
             throw;
         }
     }
-    catch (exception& e) {
+    catch (const exception& e) {
         request.log(SPRequest::SPError, e.what());
         Session* session = nullptr;
         try {
             session = request.getSession(false, true, false);   // do not cache
         }
-        catch (exception&) {
+        catch (const exception&) {
         }
         Locker slocker(session, false); // pop existing lock on exit
         TemplateParameters tp(&e, nullptr, session);
