@@ -77,7 +77,7 @@ namespace shibsp {
     class XMLExtractorImpl : public ObservableMetadataProvider::Observer
     {
     public:
-        XMLExtractorImpl(const DOMElement* e, Category& log);
+        XMLExtractorImpl(const DOMElement* e, Category& log, bool deprecationSupport);
         ~XMLExtractorImpl() {
             for (map<const ObservableMetadataProvider*,decoded_t>::iterator i=m_decodedMap.begin(); i!=m_decodedMap.end(); ++i) {
                 i->first->removeObserver(this);
@@ -102,6 +102,19 @@ namespace shibsp {
             d.clear();
         }
 
+        void onEvent(const ObservableMetadataProvider& metadata, const EntityDescriptor& entity) const {
+            // Destroy attributes we cached from this provider and entity.
+            m_attrLock->wrlock();
+            SharedLock wrapper(m_attrLock, false);
+            decoded_t& d = m_decodedMap[&metadata];
+            decoded_t::iterator i = d.find(entity.getEntityID());
+            if (i != d.end()) {
+                for_each(i->second.begin(), i->second.end(), mem_fun_ref<DDF&, DDF>(&DDF::destroy));
+                d.erase(i);
+            }
+        }
+
+
         void extractAttributes(const Application&, const char*, const char*, const NameIdentifier&, ptr_vector<Attribute>&) const;
         void extractAttributes(const Application&, const char*, const char*, const NameID&, ptr_vector<Attribute>&) const;
         void extractAttributes(const Application&, const GenericRequest*, const char*, const char*, const saml1::Attribute&, ptr_vector<Attribute>&) const;
@@ -135,29 +148,26 @@ namespace shibsp {
 
         // manages caching of decoded Attributes
         scoped_ptr<RWLock> m_attrLock;
-        typedef map< const EntityAttributes*,vector<DDF> > decoded_t;
+        typedef map< xstring,vector<DDF> > decoded_t;
         mutable map<const ObservableMetadataProvider*,decoded_t> m_decodedMap;
     };
 
     class XMLExtractor : public AttributeExtractor, public ReloadableXMLFile
     {
     public:
-        XMLExtractor(const DOMElement* e) : ReloadableXMLFile(e, Category::getInstance(SHIBSP_LOGCAT ".AttributeExtractor.XML")) {
+        XMLExtractor(const DOMElement* e, bool deprecationSupport=true)
+            : ReloadableXMLFile(e, Category::getInstance(SHIBSP_LOGCAT ".AttributeExtractor.XML"), true, deprecationSupport),
+                m_deprecationSupport(deprecationSupport)
+        {
             if (m_local && m_lock)
                 m_log.warn("attribute mappings are reloadable; be sure to restart web server when adding new attribute IDs");
             background_load();
         }
+
         ~XMLExtractor() {
             shutdown();
         }
 
-        // deprecated method
-        void extractAttributes(
-            const Application& application, const RoleDescriptor* issuer, const XMLObject& xmlObject, vector<Attribute*>& attributes
-            ) const {
-            extractAttributes(application, nullptr, issuer, xmlObject, attributes);
-        }
-
         void extractAttributes(const Application&, const GenericRequest*, const RoleDescriptor*, const XMLObject&, vector<Attribute*>&) const;
 
         void getAttributeIds(std::vector<std::string>& attributes) const {
@@ -174,6 +184,7 @@ namespace shibsp {
         pair<bool,DOMElement*> background_load();
 
     private:
+        bool m_deprecationSupport;
         scoped_ptr<XMLExtractorImpl> m_impl;
 
         void extractAttributes(const Application&, const GenericRequest*, const RoleDescriptor*, const XMLObject&, ptr_vector<Attribute>&) const;
@@ -183,7 +194,7 @@ namespace shibsp {
     #pragma warning( pop )
 #endif
 
-    AttributeExtractor* SHIBSP_DLLLOCAL XMLAttributeExtractorFactory(const DOMElement* const & e)
+    AttributeExtractor* SHIBSP_DLLLOCAL XMLAttributeExtractorFactory(const DOMElement* const & e, bool)
     {
         return new XMLExtractor(e);
     }
@@ -203,12 +214,12 @@ namespace shibsp {
     static const XMLCh _type[] =                    UNICODE_LITERAL_4(t,y,p,e);
 };
 
-XMLExtractorImpl::XMLExtractorImpl(const DOMElement* e, Category& log)
+XMLExtractorImpl::XMLExtractorImpl(const DOMElement* e, Category& log, bool deprecationSupport)
     : m_log(log),
         m_document(nullptr),
         m_policyId(XMLHelper::getAttrString(e, nullptr, metadataPolicyId)),
         m_entityAssertions(true),
-        m_metaAttrCaching(XMLHelper::getAttrBool(e, true, metadataAttributeCaching))
+        m_metaAttrCaching(XMLHelper::getAttrBool(e, false, metadataAttributeCaching))
 {
 #ifdef _DEBUG
     xmltooling::NDC ndc("XMLExtractorImpl");
@@ -224,10 +235,10 @@ XMLExtractorImpl::XMLExtractorImpl(const DOMElement* e, Category& log)
             if (t.empty())
                 throw ConfigurationException("MetadataProvider element missing type attribute.");
             m_log.info("building MetadataProvider of type %s...", t.c_str());
-            m_metadata.reset(SAMLConfig::getConfig().MetadataProviderManager.newPlugin(t.c_str(), child));
+            m_metadata.reset(SAMLConfig::getConfig().MetadataProviderManager.newPlugin(t.c_str(), child, deprecationSupport));
             m_metadata->init();
         }
-        catch (std::exception& ex) {
+        catch (const std::exception& ex) {
             m_metadata.reset();
             m_entityAssertions = false;
             m_log.crit("error building/initializing dedicated MetadataProvider: %s", ex.what());
@@ -243,9 +254,9 @@ XMLExtractorImpl::XMLExtractorImpl(const DOMElement* e, Category& log)
                 if (t.empty())
                     throw ConfigurationException("TrustEngine element missing type attribute.");
                 m_log.info("building TrustEngine of type %s...", t.c_str());
-                m_trust.reset(XMLToolingConfig::getConfig().TrustEngineManager.newPlugin(t.c_str(), child));
+                m_trust.reset(XMLToolingConfig::getConfig().TrustEngineManager.newPlugin(t.c_str(), child, deprecationSupport));
             }
-            catch (std::exception& ex) {
+            catch (const std::exception& ex) {
                 m_entityAssertions = false;
                 m_log.crit("error building/initializing dedicated TrustEngine: %s", ex.what());
                 m_log.crit("disabling support for Assertions in EntityAttributes extension");
@@ -261,9 +272,9 @@ XMLExtractorImpl::XMLExtractorImpl(const DOMElement* e, Category& log)
                 if (t.empty())
                     throw ConfigurationException("AttributeFilter element missing type attribute.");
                 m_log.info("building AttributeFilter of type %s...", t.c_str());
-                m_filter.reset(SPConfig::getConfig().AttributeFilterManager.newPlugin(t.c_str(), child));
+                m_filter.reset(SPConfig::getConfig().AttributeFilterManager.newPlugin(t.c_str(), child, deprecationSupport));
             }
-            catch (std::exception& ex) {
+            catch (const std::exception& ex) {
                 m_entityAssertions = false;
                 m_log.crit("error building/initializing dedicated AttributeFilter: %s", ex.what());
                 m_log.crit("disabling support for Assertions in EntityAttributes extension");
@@ -297,14 +308,14 @@ XMLExtractorImpl::XMLExtractorImpl(const DOMElement* e, Category& log)
         try {
             DOMElement* dchild = XMLHelper::getFirstChildElement(child, shibspconstants::SHIB2ATTRIBUTEMAP_NS, _AttributeDecoder);
             if (dchild) {
-                auto_ptr<xmltooling::QName> q(XMLHelper::getXSIType(dchild));
-                if (q.get())
-                    decoder.reset(SPConfig::getConfig().AttributeDecoderManager.newPlugin(*q.get(), dchild));
+                scoped_ptr<xmltooling::QName> q(XMLHelper::getXSIType(dchild));
+                if (q)
+                    decoder.reset(SPConfig::getConfig().AttributeDecoderManager.newPlugin(*q, dchild, deprecationSupport));
             }
             if (!decoder)
-                decoder.reset(SPConfig::getConfig().AttributeDecoderManager.newPlugin(StringAttributeDecoderType, nullptr));
+                decoder.reset(SPConfig::getConfig().AttributeDecoderManager.newPlugin(StringAttributeDecoderType, nullptr, deprecationSupport));
         }
-        catch (std::exception& ex) {
+        catch (const std::exception& ex) {
             m_log.error("skipping Attribute (%s), error building AttributeDecoder: %s", id.get(), ex.what());
         }
 
@@ -354,20 +365,22 @@ XMLExtractorImpl::XMLExtractorImpl(const DOMElement* e, Category& log)
             m_requestedAttrs.push_back(boost::tuple<xstring,xstring,bool>(entryKey.first, entryKey.second, required));
         }
 
-        name = child->getAttributeNS(nullptr, _aliases);
-        if (name && *name) {
-            m_log.warn("attribute mapping rule (%s) uses deprecated aliases feature, consider revising", id.get());
-            auto_ptr_char aliases(name);
-            string dup(aliases.get());
-            set<string> new_aliases;
-            split(new_aliases, dup, is_space(), algorithm::token_compress_on);
-            set<string>::iterator ru = new_aliases.find("REMOTE_USER");
-            if (ru != new_aliases.end()) {
-                m_log.warn("skipping alias, REMOTE_USER is a reserved name");
-                new_aliases.erase(ru);
+        if (deprecationSupport) {
+            name = child->getAttributeNS(nullptr, _aliases);
+            if (name && *name) {
+                m_log.warn("DEPRECATED: attribute mapping rule (%s) uses deprecated aliases feature", id.get());
+                auto_ptr_char aliases(name);
+                string dup(aliases.get());
+                set<string> new_aliases;
+                split(new_aliases, dup, is_space(), algorithm::token_compress_on);
+                set<string>::iterator ru = new_aliases.find("REMOTE_USER");
+                if (ru != new_aliases.end()) {
+                    m_log.warn("skipping alias, REMOTE_USER is a reserved name");
+                    new_aliases.erase(ru);
+                }
+                decl.second.insert(decl.second.end(), new_aliases.begin(), new_aliases.end());
+                m_attributeIds.insert(m_attributeIds.end(), new_aliases.begin(), new_aliases.end());
             }
-            decl.second.insert(decl.second.end(), new_aliases.begin(), new_aliases.end());
-            m_attributeIds.insert(m_attributeIds.end(), new_aliases.begin(), new_aliases.end());
         }
 
         child = XMLHelper::getNextSiblingElement(child, shibspconstants::SHIB2ATTRIBUTEMAP_NS, saml1::Attribute::LOCAL_NAME);
@@ -431,7 +444,7 @@ void XMLExtractorImpl::extractAttributes(
     }
     else if (m_log.isDebugEnabled()) {
         auto_ptr_char temp(format);
-        m_log.debug("skipping unmapped NameIdentifier with format (%s)", temp.get());
+        m_log.debug("skipping NameIdentifier with format (%s)", temp.get());
     }
 }
 
@@ -456,7 +469,7 @@ void XMLExtractorImpl::extractAttributes(
     }
     else if (m_log.isDebugEnabled()) {
         auto_ptr_char temp(format);
-        m_log.debug("skipping unmapped NameID with format (%s)", temp.get());
+        m_log.debug("skipping NameID with format (%s)", temp.get());
     }
 }
 
@@ -486,7 +499,7 @@ void XMLExtractorImpl::extractAttributes(
     else if (m_log.isInfoEnabled()) {
         auto_ptr_char temp1(name);
         auto_ptr_char temp2(format);
-        m_log.info("skipping unmapped SAML 1.x Attribute with Name: %s%s%s", temp1.get(), *temp2.get() ? ", Namespace:" : "", temp2.get());
+        m_log.info("skipping SAML 1.x Attribute with Name: %s%s%s", temp1.get(), *temp2.get() ? ", Namespace:" : "", temp2.get());
     }
 }
 
@@ -531,7 +544,7 @@ void XMLExtractorImpl::extractAttributes(
     if (m_log.isInfoEnabled()) {
         auto_ptr_char temp1(name);
         auto_ptr_char temp2(format);
-        m_log.info("skipping unmapped SAML 2.0 Attribute with Name: %s%s%s", temp1.get(), *temp2.get() ? ", Format:" : "", temp2.get());
+        m_log.info("skipping SAML 2.0 Attribute with Name: %s%s%s", temp1.get(), *temp2.get() ? ", Format:" : "", temp2.get());
     }
 }
 
@@ -581,17 +594,22 @@ void XMLExtractorImpl::extractAttributes(
     ptr_vector<Attribute>& attributes
     ) const
 {
+
+    const XMLCh* cacheID = nullptr;
+    if (observable && m_metaAttrCaching && dynamic_cast<const EntityDescriptor*>(ext.getParent())) {
+        cacheID = dynamic_cast<const EntityDescriptor*>(ext.getParent())->getEntityID();
+    }
+
     const vector<XMLObject*>& exts = ext.getUnknownXMLObjects();
     for (vector<XMLObject*>::const_iterator i = exts.begin(); i != exts.end(); ++i) {
         const EntityAttributes* container = dynamic_cast<const EntityAttributes*>(*i);
         if (!container)
             continue;
 
-        bool useCache = false;
         map<const ObservableMetadataProvider*,decoded_t>::iterator cacheEntry;
 
         // Check for cached result.
-        if (observable && m_metaAttrCaching) {
+        if (cacheID) {
             m_attrLock->rdlock();
             cacheEntry = m_decodedMap.find(observable);
             if (cacheEntry == m_decodedMap.end()) {
@@ -615,12 +633,11 @@ void XMLExtractorImpl::extractAttributes(
                     m_attrLock->rdlock();
                 }
             }
-            useCache = true;
         }
 
-        if (useCache) {
+        if (cacheID) {
             // We're holding the lock, so check the cache.
-            decoded_t::iterator d = cacheEntry->second.find(container);
+            decoded_t::iterator d = cacheEntry->second.find(cacheID);
             if (d != cacheEntry->second.end()) {
                 SharedLock locker(m_attrLock, false);   // pop the lock when we're done
                 for (vector<DDF>::iterator obj = d->second.begin(); obj != d->second.end(); ++obj) {
@@ -634,7 +651,7 @@ void XMLExtractorImpl::extractAttributes(
         }
 
         // Add a guard for the lock if we're caching.
-        SharedLock locker(useCache ? m_attrLock.get() : nullptr, false);
+        SharedLock locker(cacheID ? m_attrLock.get() : nullptr, false);
 
         // Use a holding area to support caching.
         ptr_vector<Attribute> holding;
@@ -811,14 +828,14 @@ void XMLExtractorImpl::extractAttributes(
         }
 
         if (!holding.empty()) {
-            if (useCache) {
+            if (cacheID) {
                 locker.release();   // unguard to upgrade lock
                 m_attrLock->unlock();
                 m_attrLock->wrlock();
                 SharedLock locker2(m_attrLock, false);   // pop the lock when we're done
-                if (cacheEntry->second.count(container) == 0) {
+                if (cacheEntry->second.count(cacheID) == 0) {
                     static void (vector<DDF>::* push_back)(DDF const &) = &vector<DDF>::push_back;
-                    vector<DDF>& marshalled = cacheEntry->second[container];
+                    vector<DDF>& marshalled = cacheEntry->second[cacheID];
                     for_each(
                         holding.begin(), holding.end(),
                         boost::bind(push_back, boost::ref(marshalled), boost::bind(&Attribute::marshall, _1))
@@ -1035,7 +1052,7 @@ pair<bool,DOMElement*> XMLExtractor::background_load()
     // If we own it, wrap it.
     XercesJanitor<DOMDocument> docjanitor(raw.first ? raw.second->getOwnerDocument() : nullptr);
 
-    scoped_ptr<XMLExtractorImpl> impl(new XMLExtractorImpl(raw.second, m_log));
+    scoped_ptr<XMLExtractorImpl> impl(new XMLExtractorImpl(raw.second, m_log, m_deprecationSupport));
 
     // If we held the document, transfer it to the impl. If we didn't, it's a no-op.
     impl->setDocument(docjanitor.release());
