@@ -74,7 +74,7 @@ namespace shibsp {
     class SHIBSP_DLLLOCAL SAML2SessionInitiator : public SessionInitiator, public AbstractHandler, public RemotedHandler
     {
     public:
-        SAML2SessionInitiator(const DOMElement* e, const char* appId);
+        SAML2SessionInitiator(const DOMElement* e, const char* appId, bool deprecationSupport);
         virtual ~SAML2SessionInitiator() {}
 
         void init(const char* location);    // encapsulates actions that need to run either in the c'tor or setParent
@@ -111,10 +111,12 @@ namespace shibsp {
             const char* NameIDFormat,
             const char* SPNameQualifier,
             const char* requestTemplate,
+            const char* outgoingBinding,
             string& relayState
             ) const;
 
         string m_appId;
+        bool m_deprecationSupport;
         auto_ptr_char m_paosNS,m_ecpNS;
         auto_ptr_XMLCh m_paosBinding;
 #ifndef SHIBSP_LITE
@@ -134,27 +136,23 @@ namespace shibsp {
     class SHIBSP_DLLLOCAL SessionInitiatorNodeFilter : public DOMNodeFilter
     {
     public:
-#ifdef SHIBSP_XERCESC_SHORT_ACCEPTNODE
-        short
-#else
-        FilterAction
-#endif
-        acceptNode(const DOMNode* node) const {
+        FilterAction acceptNode(const DOMNode* node) const {
             return FILTER_REJECT;
         }
     };
 
     static SHIBSP_DLLLOCAL SessionInitiatorNodeFilter g_SINFilter;
 
-    SessionInitiator* SHIBSP_DLLLOCAL SAML2SessionInitiatorFactory(const pair<const DOMElement*,const char*>& p)
+    SessionInitiator* SHIBSP_DLLLOCAL SAML2SessionInitiatorFactory(const pair<const DOMElement*,const char*>& p, bool deprecationSupport)
     {
-        return new SAML2SessionInitiator(p.first, p.second);
+        return new SAML2SessionInitiator(p.first, p.second, deprecationSupport);
     }
 
 };
 
-SAML2SessionInitiator::SAML2SessionInitiator(const DOMElement* e, const char* appId)
-    : AbstractHandler(e, Category::getInstance(SHIBSP_LOGCAT ".SessionInitiator.SAML2"), &g_SINFilter, &m_remapper), m_appId(appId),
+SAML2SessionInitiator::SAML2SessionInitiator(const DOMElement* e, const char* appId, bool deprecationSupport)
+    : AbstractHandler(e, Category::getInstance(SHIBSP_LOGCAT ".SessionInitiator.SAML2"), &g_SINFilter, this),
+        m_appId(appId), m_deprecationSupport(deprecationSupport),
         m_paosNS(samlconstants::PAOS_NS), m_ecpNS(samlconstants::SAML20ECP_NS), m_paosBinding(samlconstants::SAML20_BINDING_PAOS)
 #ifdef SHIBSP_LITE
         ,m_ecp(false)
@@ -204,11 +202,7 @@ void SAML2SessionInitiator::init(const char* location)
         // If directed, build an ECP encoder.
         if (flag.first && flag.second) {
             try {
-                m_ecp.reset(
-                    SAMLConfig::getConfig().MessageEncoderManager.newPlugin(
-                        samlconstants::SAML20_BINDING_PAOS, pair<const DOMElement*,const XMLCh*>(getElement(), nullptr)
-                        )
-                    );
+                m_ecp.reset(SAMLConfig::getConfig().MessageEncoderManager.newPlugin(samlconstants::SAML20_BINDING_PAOS, getElement(), m_deprecationSupport));
             }
             catch (std::exception& ex) {
                 m_log.error("error building PAOS/ECP MessageEncoder: %s", ex.what());
@@ -229,9 +223,7 @@ void SAML2SessionInitiator::init(const char* location)
         split(m_bindings, dupBindings, is_space(), algorithm::token_compress_on);
         for (vector<string>::const_iterator b = m_bindings.begin(); b != m_bindings.end(); ++b) {
             try {
-                boost::shared_ptr<MessageEncoder> encoder(
-                    SAMLConfig::getConfig().MessageEncoderManager.newPlugin(*b, pair<const DOMElement*,const XMLCh*>(getElement(),nullptr))
-                    );
+                boost::shared_ptr<MessageEncoder> encoder(SAMLConfig::getConfig().MessageEncoderManager.newPlugin(*b, getElement(), m_deprecationSupport));
                 if (encoder->isUserAgentPresent() && XMLString::equals(getProtocolFamily(), encoder->getProtocolFamily())) {
                     m_encoders[*b] = encoder;
                     m_log.debug("supporting outgoing binding (%s)", b->c_str());
@@ -267,6 +259,7 @@ pair<bool,long> SAML2SessionInitiator::run(SPRequest& request, string& entityID,
     const Handler* ACS = nullptr;
     pair<bool,const char*> acClass, acComp, nidFormat, spQual;
     const char* requestTemplate = nullptr;
+    const char* outgoingBinding = nullptr;
     bool isPassive=false,forceAuthn=false;
     const Application& app = request.getApplication();
 
@@ -301,6 +294,8 @@ pair<bool,long> SAML2SessionInitiator::run(SPRequest& request, string& entityID,
             requestTemplate = request.getParameter("template");
         }
 
+        outgoingBinding = request.getParameter("outgoingBinding");
+
         pair<bool,bool> flag = getBool("isPassive", request, settingMask);
         isPassive = (flag.first && flag.second);
 
@@ -388,7 +383,7 @@ pair<bool,long> SAML2SessionInitiator::run(SPRequest& request, string& entityID,
             // Determine index to use.
             pair<bool,const XMLCh*> ix = pair<bool,const XMLCh*>(false,nullptr);
             if (!strncmp(ACSloc.c_str(), "https://", 8)) {
-            	ix = ACS->getXMLString("sslIndex", shibspconstants::ASCII_SHIB2SPCONFIG_NS);
+            	ix = ACS->getXMLString("sslIndex", shibspconstants::ASCII_SHIBSPCONFIG_NS);
             	if (!ix.first)
             		ix = ACS->getXMLString("index");
             }
@@ -407,6 +402,7 @@ pair<bool,long> SAML2SessionInitiator::run(SPRequest& request, string& entityID,
                 nidFormat.first ? nidFormat.second : nullptr,
                 spQual.first ? spQual.second : nullptr,
                 requestTemplate,
+                outgoingBinding,
                 target
                 );
         }
@@ -437,6 +433,7 @@ pair<bool,long> SAML2SessionInitiator::run(SPRequest& request, string& entityID,
             nidFormat.first ? nidFormat.second : nullptr,
             spQual.first ? spQual.second : nullptr,
             requestTemplate,
+            outgoingBinding,
             target
             );
     }
@@ -461,11 +458,13 @@ pair<bool,long> SAML2SessionInitiator::run(SPRequest& request, string& entityID,
         in.addmember("SPNameQualifier").string(spQual.second);
     if (requestTemplate)
         in.addmember("template").string(requestTemplate);
+    if (outgoingBinding)
+        in.addmember("outgoingBinding").string(outgoingBinding);
     if (acsByIndex.first && acsByIndex.second) {
         // Determine index to use.
         pair<bool,const char*> ix = pair<bool,const char*>(false,nullptr);
         if (!strncmp(ACSloc.c_str(), "https://", 8)) {
-        	ix = ACS->getString("sslIndex", shibspconstants::ASCII_SHIB2SPCONFIG_NS);
+        	ix = ACS->getString("sslIndex", shibspconstants::ASCII_SHIBSPCONFIG_NS);
         	if (!ix.first)
         		ix = ACS->getString("index");
         }
@@ -501,7 +500,7 @@ pair<bool,long> SAML2SessionInitiator::run(SPRequest& request, string& entityID,
         in.addmember("RelayState").unsafe_string(target.c_str());
 
     // Remote the processing.
-    out = request.getServiceProvider().getListenerService()->send(in);
+    out = send(request, in);
     return unwrap(request, out);
 }
 
@@ -530,7 +529,7 @@ void SAML2SessionInitiator::receive(DDF& in, ostream& out)
     DDFJanitor jout(ret);
 
     // Wrap the outgoing object with a Response facade.
-    scoped_ptr<HTTPResponse> http(getResponse(ret));
+    scoped_ptr<HTTPResponse> http(getResponse(*app, ret));
 
     auto_ptr_XMLCh index(in["acsIndex"].string());
     auto_ptr_XMLCh bind(in["acsBinding"].string());
@@ -553,6 +552,7 @@ void SAML2SessionInitiator::receive(DDF& in, ostream& out)
         in["NameIDFormat"].string(),
         in["SPNameQualifier"].string(),
         in["template"].string(),
+        in["outgoingBinding"].string(),
         relayState
         );
     if (!ret.isstruct())
@@ -577,6 +577,7 @@ pair<bool,long> SAML2SessionInitiator::doRequest(
     const char* NameIDFormat,
     const char* SPNameQualifier,
     const char* requestTemplate,
+    const char* outgoingBinding,
     string& relayState
     ) const
 {
@@ -623,6 +624,8 @@ pair<bool,long> SAML2SessionInitiator::doRequest(
         // Loop over the supportable outgoing bindings.
         role = dynamic_cast<const IDPSSODescriptor*>(entity.second);
         for (vector<string>::const_iterator b = m_bindings.begin(); b != m_bindings.end(); ++b) {
+            if (outgoingBinding && *b != outgoingBinding)
+                continue;
             auto_ptr_XMLCh wideb(b->c_str());
             ep = EndpointManager<SingleSignOnService>(role->getSingleSignOnServices()).getByBinding(wideb.get());
             if (ep) {
@@ -648,15 +651,11 @@ pair<bool,long> SAML2SessionInitiator::doRequest(
     auto_ptr<AuthnRequest> req;
 
     if (requestTemplate) {
-        xsecsize_t x;
+        XMLSize_t x;
         XMLByte* decoded=Base64::decode(reinterpret_cast<const XMLByte*>(requestTemplate), &x);
         if (decoded) {
             istringstream is(reinterpret_cast<char*>(decoded));
-#ifdef SHIBSP_XERCESC_HAS_XMLBYTE_RELEASE
-            XMLString::release(&decoded);
-#else
             XMLString::release((char**)&decoded);
-#endif
             DOMDocument* doc = XMLToolingConfig::getConfig().getParser().parse(is);
             XercesJanitor<DOMDocument> docjanitor(doc);
             auto_ptr<XMLObject> xmlObject(XMLObjectBuilder::buildOneFromElement(doc->getDocumentElement(), true));
